@@ -1,8 +1,8 @@
 """Trainer and configuration for MBPO."""
-from ray.rllib.utils import override
-
 from raylab.agents.model_based import ModelBasedTrainer
 from raylab.agents.model_based import with_base_config
+from raylab.policy.model_based.sampling_mixin import SamplingSpec
+from raylab.policy.model_based.training_mixin import TrainingSpec
 
 from .policy import MBPOTorchPolicy
 
@@ -16,43 +16,32 @@ DEFAULT_CONFIG = with_base_config(
         # === Twin Delayed DDPG (TD3) tricks ===
         # Clipped Double Q-Learning
         "clipped_double_q": True,
-        # === Model Training ===
+        # === ModelBasedTrainer ===
         # Fraction of input data for validation and early stopping, may be 0.
         "holdout_ratio": 0.2,
         # Maximum number of samples for validation and early stopping
         "max_holdout": 5000,
-        # Maximum number of full model passes through the data, may be None.
-        "max_model_epochs": None,
-        # Maximum number of model gradient steps, may be None.
-        "max_model_steps": 120,
-        # Minimum expected relative improvement in model validation loss
-        "improvement_threshold": 0.01,
-        # Number of epochs to wait for any of the models to improve on the validation
-        # dataset before early stopping
-        "patience_epochs": 5,
-        # Maximum time in seconds for training the model, may be None.
-        # We check this after each epoch (not minibatch)
-        "max_model_train_s": 20,
-        # Size of minibatch for dynamics model training
-        "model_batch_size": 256,
-        # === Policy Training ===
+        # Size of the buffer for virtual samples
+        "virtual_buffer_size": int(1e5),
+        # Number of model rollouts to add to augmented replay per real environment step
+        "model_rollouts": 40,
         # Number of policy improvement steps per real environment step
         "policy_improvements": 10,
         # Fraction of each policy minibatch to sample from environment replay pool
         "real_data_ratio": 0.1,
         "train_batch_size": 512,
+        # Wait until this many steps have been sampled before starting optimization.
+        "learning_starts": 10000,
+        # === Policy mixins ===
+        # Specifications for model training
+        # See `raylab.policy.model_based.training_mixin`
+        "model_training": TrainingSpec().to_dict(),
+        # Specifications for model sampling
+        # See `raylab.policy.model_based.sampling_mixin`
+        "model_sampling": SamplingSpec().to_dict(),
         # === Replay buffer ===
         # Size of the replay buffer.
         "buffer_size": int(1e5),
-        # Size of the buffer for virtual samples
-        "virtual_buffer_size": int(1e5),
-        # Number of model rollouts to add to augmented replay per real environment step
-        "model_rollouts": 40,
-        # Lenght of model-based rollouts from each state sampled from replay
-        "model_rollout_length": 1,
-        # Use this number of best performing models on the validation dataset to sample
-        # transitions
-        "num_elites": 5,
         # === Optimization ===
         # PyTorch optimizers to use
         "torch_optimizer": {
@@ -63,8 +52,6 @@ DEFAULT_CONFIG = with_base_config(
         },
         # Interpolation factor in polyak averaging for target networks.
         "polyak": 0.995,
-        # Wait until this many steps have been sampled before starting optimization.
-        "learning_starts": 10000,
         # === Network ===
         "module": {
             "type": "ModelBasedSAC",
@@ -103,28 +90,3 @@ class MBPOTrainer(ModelBasedTrainer):
     _name = "MBPO"
     _default_config = DEFAULT_CONFIG
     _policy = MBPOTorchPolicy
-
-    @staticmethod
-    @override(ModelBasedTrainer)
-    def validate_config(config):
-        ModelBasedTrainer.validate_config(config)
-        assert (
-            config["max_model_epochs"] is None or config["max_model_epochs"] >= 0
-        ), "Cannot train model for a negative number of epochs"
-        assert (
-            config["patience_epochs"] > 0
-        ), "Must wait a positive number of epochs for any model to improve"
-        assert config["model_batch_size"] > 0, "Model batch size must be positive"
-
-        assert (
-            config["model_rollout_length"] > 0
-        ), "Length of model-based rollouts must be positive"
-
-        stopping_criteria = (
-            config["holdout_ratio"] > 0
-            or config["max_model_epochs"]
-            or config["max_model_train_s"]
-        )
-        assert (
-            stopping_criteria
-        ), "MBPO needs at least one stopping criteria for model training"
