@@ -24,10 +24,7 @@ class SVGInfTorchPolicy(AdaptiveKLCoeffMixin, SVGTorchPolicy):
         self._off_policy_learning = False
 
         self.loss_actor = TrajectorySVG(
-            self.module.model,
-            self.module.actor,
-            self.module.critic,
-            torch_script=self.config["module"].get("torch_script", False),
+            self.module.model, self.module.actor, self.module.critic,
         )
 
     @override(EnvFnMixin)
@@ -58,6 +55,11 @@ class SVGInfTorchPolicy(AdaptiveKLCoeffMixin, SVGTorchPolicy):
             for name, module in component_map.items()
         }
 
+    @override(SVGTorchPolicy)
+    def compile(self):
+        super().compile()
+        self.loss_actor.compile()
+
     @override(SVGTorchPolicy)
     def learn_on_batch(self, samples):
         batch_tensors = self.lazy_tensor_dict(samples)
