@@ -1,44 +1,34 @@
 """Modularized Q-Learning procedures."""
+from typing import Callable
+
 import torch
 import torch.nn as nn
 from ray.rllib import SampleBatch
-from ray.rllib.utils import override
+from torch import Tensor
 
 import raylab.utils.dictionaries as dutil
+from raylab.utils.annotations import DetPolicy
+from raylab.utils.annotations import StochasticPolicy
 
+from .abstract import Loss
 from .utils import clipped_action_value
 
 
-class ClippedDoubleQLearning:
-    """Clipped Double Q-Learning.
+class QLearningMixin:
+    """Adds default call for Q-Learning losses."""
 
-    Use the minimun of two target Q functions as the next action-value in the target
-    for fitted Q iteration.
-
-    Args:
-        critics (list): callables for main action-values
-        target_critics (list): callables for target action-values
-        actor (callable): deterministic policy for the next state
-        gamma (float): discount factor
-    """
-
-    def __init__(self, critics, target_critics, actor, gamma):
-        self.critics = critics
-        self.target_critics = target_critics
-        self.actor = actor
-        self.gamma = gamma
+    # pylint:disable=too-few-public-methods
+    batch_keys = (
+        SampleBatch.CUR_OBS,
+        SampleBatch.ACTIONS,
+        SampleBatch.REWARDS,
+        SampleBatch.NEXT_OBS,
+        SampleBatch.DONES,
+    )
 
     def __call__(self, batch):
         """Compute loss for Q-value function."""
-        # pylint:disable=too-many-arguments
-        obs, actions, rewards, next_obs, dones = dutil.get_keys(
-            batch,
-            SampleBatch.CUR_OBS,
-            SampleBatch.ACTIONS,
-            SampleBatch.REWARDS,
-            SampleBatch.NEXT_OBS,
-            SampleBatch.DONES,
-        )
+        obs, actions, rewards, next_obs, dones = dutil.get_keys(batch, *self.batch_keys)
         with torch.no_grad():
             target_values = self.critic_targets(rewards, next_obs, dones)
         loss_fn = nn.MSELoss()
@@ -53,6 +43,32 @@ class ClippedDoubleQLearning:
         }
         return critic_loss, stats
 
+
+class ClippedDoubleQLearning(QLearningMixin, Loss):
+    """Clipped Double Q-Learning.
+
+    Use the minimun of two target Q functions as the next action-value in the target
+    for fitted Q iteration.
+
+    Args:
+        critics: callables for main action-values
+        target_critics: callables for target action-values
+        actor: deterministic policy for the next state
+        gamma: discount factor
+    """
+
+    def __init__(
+        self,
+        critics: nn.ModuleList,
+        target_critics: nn.ModuleList,
+        actor: DetPolicy,
+        gamma: float,
+    ):
+        self.critics = critics
+        self.target_critics = target_critics
+        self.actor = actor
+        self.gamma = gamma
+
     def critic_targets(self, rewards, next_obs, dones):
         """
         Compute 1-step approximation of Q^{\\pi}(s, a) for Clipped Double Q-Learning
@@ -64,26 +80,37 @@ class ClippedDoubleQLearning:
         return rewards + self.gamma * next_values
 
 
-class SoftCDQLearning(ClippedDoubleQLearning):
+class SoftCDQLearning(QLearningMixin, Loss):
     """Clipped Double Q-Learning for maximum entropy RL.
 
     Args:
-        critics (list): callables for main action-values
-        target_critics (list): callables for target action-values
-        actor (callable): stochastic policy for the next state
-        gamma (float): discount factor
-        alpha (callable): entropy coefficient schedule
+        critics: callables for main action-values
+        target_critics: callables for target action-values
+        actor: stochastic policy for the next state
+        gamma: discount factor
+        alpha: entropy coefficient schedule
     """
 
-    # pylint:disable=too-few-public-methods
-
-    def __init__(self, critics, target_critics, actor, gamma, alpha):
+    def __init__(
+        self,
+        critics: nn.ModuleList,
+        target_critics: nn.ModuleList,
+        actor: StochasticPolicy,
+        gamma: float,
+        alpha: Callable[[], Tensor],
+    ):
         # pylint:disable=too-many-arguments
-        super().__init__(critics, target_critics, actor.sample, gamma)
+        self.critics = critics
+        self.target_critics = target_critics
+        self.actor = actor
+        self.gamma = gamma
         self.alpha = alpha
 
-    @override(ClippedDoubleQLearning)
     def critic_targets(self, rewards, next_obs, dones):
+        """
+        Compute 1-step approximation of Q^{\\pi}(s, a) for Clipped Double Q-Learning
+        using target networks and batch transitions.
+        """
         next_acts, next_logp = self.actor(next_obs)
         target_values = clipped_action_value(next_obs, next_acts, self.target_critics)
 
